@@ -41,6 +41,28 @@ DeclarationNavigationContext::DeclarationNavigationContext( DeclarationPointer d
 {
 }
 
+QualifiedIdentifier DeclarationNavigationContext::prettyQualifiedIdentifier(DeclarationPointer decl) const
+{
+  QualifiedIdentifier ret;
+  if(m_topContext.data() && decl) {
+      if(decl->kind() == Declaration::Type) {
+        AbstractType::Ptr type = decl->abstractType();
+        DelayedType::Ptr stripped = stripType(type, m_topContext.data()).cast<DelayedType>();
+        if(stripped)
+          return stripped->identifier().identifier().identifier();
+        else
+          return decl->qualifiedIdentifier();
+      }else{
+        if(decl->context()->owner()) {
+          return prettyQualifiedIdentifier(DeclarationPointer(decl->context()->owner())) + decl->identifier();
+        }else{
+          return decl->qualifiedIdentifier();
+        }
+      }
+  }
+  return ret;
+}
+
 KDevelop::AbstractType::Ptr DeclarationNavigationContext::typeToShow(KDevelop::AbstractType::Ptr type) {
   return shortenTypeForViewing(type);
 }
@@ -61,11 +83,31 @@ void DeclarationNavigationContext::htmlIdentifiedType(AbstractType::Ptr type, co
 
   if( const TemplateDeclaration* templ = dynamic_cast<const TemplateDeclaration*>(idType->declaration(m_topContext.data())) ) {
     if( templ->instantiatedFrom() ) {
-      modifyHtml() += Qt::escape("  <");
 
       const Cpp::InstantiationInformation& params = templ->instantiatedWith().information();
+      
+      if(params.templateParametersSize() == 0)
+        return;
+      
+      modifyHtml() += Qt::escape("  <");
       bool first = true;
-      FOREACH_FUNCTION( const IndexedType& type, params.templateParameters ) {
+      
+      uint showParameters = params.templateParametersSize();
+      
+      if(m_topContext.data()) {
+        //Strip away default-parameters, as they can be overwhelmingly ugly, especially in STL
+        DelayedType::Ptr stripped = stripType(type, m_topContext.data()).cast<DelayedType>();
+        if(stripped) {
+            QualifiedIdentifier id = stripped->identifier().identifier().identifier();
+            if(!id.isEmpty() && id.last().templateIdentifiersCount() < showParameters)
+              showParameters = id.last().templateIdentifiersCount();
+        }
+      }
+      
+      
+      for( uint p = 0; p < showParameters; ++p ) {
+      const IndexedType& type(params.templateParameters()[p]);
+      
         if( first )
           first = false;
         else
