@@ -48,6 +48,7 @@
 #include <language/duchain/classdeclaration.h>
 #include <cppducontext.h>
 #include <interfaces/iassistant.h>
+#include <language/duchain/use.h>
 
 QTEST_KDEMAIN(TestCppCodegen, GUI )
 
@@ -583,4 +584,24 @@ void TestCppCodegen::testPrivateImplementation()
 #endif
 }
 
+void TestCppCodegen::testMacroDeclarationOrder()
+{
+  InsertIntoDUChain source("thefile.h",
+    "#define DECLARE_MY_ITERATOR\
+     template <class Key, class T> class MyIteratorMacro { T value(); Key key(); };\n\
+     DECLARE_MY_ITERATOR\n\
+     template <class Key, class T> class MyIteratorDirect { Key key(); T value(); };");
+  source.parse(TopDUContext::AllDeclarationsContextsUsesAndAST);
+  DUChainReadLocker lock;
+  ReferencedTopDUContext top = source.m_topContext;
+  Declaration* macroKey = top->findDeclarations(QualifiedIdentifier("MyIteratorMacro<int,char>::key")).first();
+  Declaration* macroVal = top->findDeclarations(QualifiedIdentifier("MyIteratorMacro<int,char>::value")).first();
+  Declaration* directKey = top->findDeclarations(QualifiedIdentifier("MyIteratorDirect<int,char>::key")).first();
+  Declaration* directVal = top->findDeclarations(QualifiedIdentifier("MyIteratorDirect<int,char>::value")).first();
+  QVERIFY(directKey->abstractType()->toString() == "function int ()");
+  QVERIFY(directVal->abstractType()->toString() == "function char ()");
+  QVERIFY(macroKey->abstractType()->toString() == "function int ()");
+  QVERIFY(macroVal->abstractType()->toString() == "function char ()");
+}
+
 #include "test_cppcodegen.moc"
