@@ -336,6 +336,7 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
         connect(w, SIGNAL(deleted(QString)), this, SLOT(deletedWatched(QString)));
         m_watchers[project] = w;
         Q_ASSERT(m_rootItem->rowCount()==0);
+        cfg->sync();
     }
     return m_rootItem;
 }
@@ -366,6 +367,7 @@ QSet<QString> filterFiles(const QStringList& orig)
 
 QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolderItem* item )
 {
+    Q_ASSERT(isReloading(item->project()));
     QList<KDevelop::ProjectFolderItem*> folderList;
     CMakeFolderItem* folder = dynamic_cast<CMakeFolderItem*>( item );
 
@@ -690,11 +692,13 @@ bool CMakeManager::reload(KDevelop::ProjectFolderItem* folder)
 
 void CMakeManager::reimport(KDevelop::ProjectFolderItem* fi)
 {
+    Q_ASSERT(!isReloading(fi->project()));
     KJob *job=createImportJob(fi);
     job->setProperty("projectitem", qVariantFromValue(fi));
     
     QMutexLocker locker(&m_busyProjectsMutex);
-    m_busyProjects += fi;
+    Q_ASSERT(!m_busyProjects.contains(fi->project()));
+    m_busyProjects += fi->project();
     locker.unlock();
     
     connect( job, SIGNAL( result( KJob* ) ), this, SLOT( reimportDone( KJob* ) ) );
@@ -706,8 +710,8 @@ void CMakeManager::reimportDone(KJob* job)
     QMutexLocker locker(&m_busyProjectsMutex);
     ProjectFolderItem* it = job->property("projectitem").value<KDevelop::ProjectFolderItem*>();
     
-    Q_ASSERT(m_busyProjects.contains(it));
-    m_busyProjects.remove(it);
+    Q_ASSERT(m_busyProjects.contains(it->project()));
+    m_busyProjects.remove(it->project());
 }
 
 bool CMakeManager::isReloading(IProject* p)
@@ -716,23 +720,28 @@ bool CMakeManager::isReloading(IProject* p)
         return true;
     
     QMutexLocker locker(&m_busyProjectsMutex);
-    foreach(KDevelop::ProjectFolderItem* it, m_busyProjects) {
-        if(it->project()==p)
-            return true;
-    }
-    return false;
+    
+    return m_busyProjects.contains(p);
 }
 
 void CMakeManager::deletedWatched(const QString& path)
 {
     KUrl dirurl(path);
-    IProject* p=ICore::self()->projectController()->findProjectForUrl(dirurl);
+    IProject* p=0;
+    
+    QMutexLocker locker(&m_busyProjectsMutex);
+    if(m_busyProjects.isEmpty())
+        p=ICore::self()->projectController()->findProjectForUrl(dirurl);
+    else
+        QMetaObject::invokeMethod(this, "deletedWatched", Qt::QueuedConnection, Q_ARG(QString, path));
+    locker.unlock();
     
     if(p) {
+        Q_ASSERT(!isReloading(p));
         dirurl.adjustPath(KUrl::AddTrailingSlash);
-        if(p->folder()==dirurl)
+        if(p->folder()==dirurl) {
             ICore::self()->projectController()->closeProject(p);
-        else if(!isReloading(p)) {
+        } else {
             KUrl url(path);
             
             if(path.endsWith("/CMakeLists.txt")) {
@@ -741,7 +750,14 @@ void CMakeManager::deletedWatched(const QString& path)
                     reload(folder);
                 
             } else {
+                QMutexLocker locker(&m_busyProjectsMutex);
+                m_busyProjects += p;
+                locker.unlock();
+                
                 qDeleteAll(p->itemsForUrl(url));
+                
+                locker.relock();
+                m_busyProjects -= p;
             }
         }
     }
@@ -811,13 +827,13 @@ void CMakeManager::dirtyFile(const QString & dirty)
         
         if(!folders.isEmpty()) {
             QMutexLocker locker(&m_busyProjectsMutex);
-            m_busyProjects += folders.first();
+            m_busyProjects += p;
             locker.unlock();
             
             reloadFiles(folders.first());
             
             locker.relock();
-            m_busyProjects.remove(folders.first());
+            m_busyProjects -= p;
             locker.unlock();
         }
     }
@@ -825,6 +841,8 @@ void CMakeManager::dirtyFile(const QString & dirty)
 
 void CMakeManager::reloadFiles(ProjectFolderItem* item)
 {
+    Q_ASSERT(isReloading(item->project()));
+    
     QDir d(item->url().toLocalFile());
     if(!d.exists()) {
         kDebug() << "Trying to return a directory that doesn't exist:" << item->url();
