@@ -23,16 +23,25 @@
 
 #include <QtTest/QtTest>
 
+class CMakeProjectVisitor;
 class CMakeCompliance : public QObject
 {
     Q_OBJECT
-public:
-    CMakeCompliance() {}
-    virtual ~CMakeCompliance() {}
+    public:
+        CMakeCompliance();
+        virtual ~CMakeCompliance() {}
+        static void addOutput(const QString& msg);
 
-private slots:
-    void testEnumerate();
-    void testEnumerate_data();
+    private slots:
+        void testEnumerate();
+        void testEnumerate_data();
+        
+        void testCMakeTests();
+        void testCMakeTests_data();
+        
+    private:
+        CMakeProjectVisitor parseFile( const QString& sourcefile );
+        static QString output;
 };
 
 #endif
